@@ -587,10 +587,17 @@ use crate::traits::NumberTheory;
   
     // weighted to maintain at most 2^-64 probability of failure, slower than most implementations for small numbers but faster for larger. Values greater than 2^512 receive only two checks, a strong-base 2 and a random-base check. This is due to the fact that the density of pseudoprimes rapidly declines
   pub fn probable_prime(&self) -> bool{
-      const CHECK_LUT : [u8;10] = [12,11,9,6,5,5,4,3,2,1];
+      const CHECK_LUT : [u8;10]  = [12,11,9,6,5,5,4,3,2,1];
+      const DIV_BOUND : [u16;10] = [380,500,800,1000,1200,1400,1600,1800,2000,2048];
       let mut check = 1;
+      let mut supremum = 2048;
+      if self.len() < 2usize {
+         return self.to_u64().unwrap().is_prime()
+      }
+      
      if self.len() < 8{
-         check = CHECK_LUT[self.len()]
+         check = CHECK_LUT[self.len()];
+         supremum = DIV_BOUND[self.len()];
      }
 
    let two = Mpz::from_u64(2);
@@ -605,7 +612,7 @@ use crate::traits::NumberTheory;
       None    => (),
     }
     
-   for i in PRIMELIST[1..].iter(){ // apparently optimal on my machine
+   for i in PRIMELIST[1..supremum as usize].iter(){ 
      if self.congruence_u64(*i as u64,0){
        return false
      }
