@@ -1,12 +1,18 @@
  
  use crate::primes::PRIMELIST;
  use crate::arithmetic::mpz::Mpz;
- 
+ use crate::arithmetic::inlineops::*;
  
  use crate::traits::NumberTheory;
  
  impl NumberTheory for u128 {
  
+  fn rng() -> Self {fuse(rng_64(), rng_64()) }
+  
+  fn euclidean(&self, other: &Self) -> (Self,Self) {
+   (*self/ *other, *self%*other)
+  }
+   
   fn is_prime(&self) -> bool{
      if *self < u64::MAX as u128{
        return (*self as u64).is_prime()
@@ -137,10 +143,32 @@
  fn mod_pow(&self, p: &Self, modulus: &Self)-> Self{  
                    if modulus == &0 {return 0u128} Mpz::from_u128(*self).u_mod_pow(&Mpz::from_u128(*p),&Mpz::from_u128(*modulus) ).to_u128().unwrap()
   }
+  
+  
+ fn legendre(&self, p: &Self) -> i8 {
+    let k = self.mod_pow(&((*p-1)>>1), p);
+    if k == 1{return 1};
+    if k == *p-1 {return -1};
+    return 0
+ }
+ 
+ fn checked_legendre(&self, p: &Self) -> Option<i8> {
+     if *p == 2 {return None}
+     match p.is_prime(){
+       true  => Some(self.legendre(p)),
+       false => None,
+     }
+ } 
 }  
 
  impl NumberTheory for i128{
  
+ fn rng() -> Self {fuse(rng_64(), rng_64()) as i128}
+ 
+ fn euclidean(&self, other: &Self) -> (Self,Self) {
+   (*self/ *other, *self%*other)
+  }
+ 
  fn is_prime(&self) -> bool{
     (self.abs() as u128).is_prime()
   }
@@ -192,6 +220,15 @@
   }
 
  
+ 
+ fn legendre(&self, p: &Self) -> i8 {
+       (self.abs() as u128).legendre(&(p.abs() as u128))
+ }
+ 
+  fn checked_legendre(&self, p: &Self) -> Option<i8> {
+     (self.abs() as u128).checked_legendre(&(p.abs() as u128))
+ }
+ 
  }
   
   
